import requests

from .channel import Channel
from .artist import Artist
from .post import Post
from .viral import Viral
from .charts import Chart

from .coupon import Coupon
from .brand import Brand
from .milestone import Milestone

from .types import TYPES

class NindoApi(object):

    def getVirals(self):

        api_response = requests.request("GET", "https://api.nindo.de/viral")

        response = []
        for post in api_response.json():

            newPost = Post(post["postID"], post["platform"], post["timestamp"], post["_post"]["title"], post["_post"]["FSK18"], post["_post"]["clickbait"], post["_post"]["shitstorm"], post["_post"]["ad"])
            newChannel = Channel(post["_post"]["_channel"]["channelID"], post["_post"]["_channel"]["avatar"], post["_post"]["_channel"]["lastPostID"])
            newArtist = Artist(post["_post"]["_channel"]["_artist"]["name"], post["_post"]["_channel"]["_artist"]["_id"])
            newViral = Viral(post["postID"], post["platform"], post["timestamp"], post["type"], post["value"])

            response.append({"post": newPost, "channel": newChannel, "artist": newArtist, "viral": newViral})

        return response

    def getCharts(self, type):

        if type == TYPES.Charts.INSTAGRAM or type == TYPES.Charts.TIKTOK or type == TYPES.Charts.TWITCH or type == TYPES.Charts.YOUTUBE or type == TYPES.Charts.TWITTER:

            api_response = requests.request("GET", f"https://api.nindo.de/ranks/charts/{type}")
            response = []

            for chart in api_response.json():

                newArtist = Artist(chart["_artist"]["name"], chart["artistID"])
                myChart = Chart(chart["id"], chart["rank"], newArtist, chart["value"], type)
                response.append(myChart)

            return response

    def getCoupons(self):

        api_response = requests.request("GET", f"https://api.nindo.de/coupons")
        response = []

        for currCoupon in api_response.json()["coupons"]:

            newArtist = Artist(currCoupon["_artist"]["name"], currCoupon["_artist"]["id"])
            newBrand = Brand(currCoupon["_brand"]["id"], currCoupon["_brand"]["name"], currCoupon["_brand"]["url"], currCoupon["_brand"]["branch"], currCoupon["_brand"]["color"])

            newCoupon = Coupon(currCoupon["id"], currCoupon["timestamp"], currCoupon["discount"], currCoupon["code"], currCoupon["valid"], currCoupon["validUntil"], currCoupon["url"], currCoupon["terms"], newArtist, newBrand)
            response.append(newCoupon)

        return response

    def getUpcomingMilestones(self):

        api_response = requests.request("GET", f"https://api.nindo.de/ranks/milestones")
        response = []

        for milestone in api_response.json():

            newChannel = Channel(milestone["_channel"]["id"], milestone["_channel"]["avatar"], None)
            newArtist = Artist(milestone["_channel"]["_artist"]["name"], milestone["_channel"]["artist_id"])
            newMilestone = Milestone(milestone["currentSubs"], milestone["expectedTime"], False, newChannel, newArtist)

            response.append(newMilestone)

        return response

    def getPastMilestones(self):

        api_response = requests.request("GET", f"https://api.nindo.de/ranks/pastMilestones")
        response = []

        for milestone in api_response.json():
            newChannel = Channel(milestone["_channel"]["id"], milestone["_channel"]["avatar"], None)
            newArtist = Artist(milestone["_channel"]["_artist"]["name"], milestone["_channel"]["artist_id"])
            newMilestone = Milestone(milestone["currentSubs"], milestone["expectedTime"],
                                     True, newChannel, newArtist)

            response.append(newMilestone)

        return response



class Channel(object):

    def __init__(self, id, avatar, last_post_id):

        self.id = id
        self.avatar = avatar
        self.last_post_id = last_post_id

    def __repr__(self):

        return f"<nindo.api.Channel '{self.id}'>"


class Post(object):

    def __init__(self, id, platform, timestamp, title, age_restricted, clickbait_score, shitstorm_score, ad_score):

        self.id = id
        self.platform = platform
        self.timestamp = timestamp
        self.title = title
        self.age_restricted = age_restricted
        self.clickbait_score = clickbait_score
        self.shitstorm_score = shitstorm_score
        self.ad_score = ad_score

        self.artist = None
        self.channel = None

    def __repr__(self):

        return f"<nindo.api.Post '{self.id}'>"


class Viral(object):

    def __init__(self, id, platform, timestamp, type, value):

        self.id = id,
        self.platform = platform
        self.timestamp = timestamp
        self.type = type
        self.value = value

    def __repr__(self):

        return f"<nindo.api.Viral '{self.id}'>"


from .types import TYPES
class Chart(object):

    def __init__(self, id, rank, artist, value, chartType):

        self.id = id
        self.rank = rank
        self.artists = artist
        self.value = value
        self.type = TYPES.Charts.DISPLAY[chartType]

    def __repr__(self):

        return f"<nindo.api.Chart '{self.id}'>"


from .brand import Brand

class Coupon(object):

    def __init__(self, id, timestamp, discount, code, valid, validUntil, url, terms, artist, brand):

        self.id = id
        self.timestamp = timestamp
        self.discount = discount
        self.code = code
        self.valid = valid
        self.validUntil = validUntil
        self.url = url
        self.terms = terms

        self.artist = artist
        self.brand = brand

    def __repr__(self):

        return f"<nindo.api.Coupon '{self.id}'>"


class TYPES(object):

    class Charts(object):

        YOUTUBE = "youtube/rankViews/big"
        INSTAGRAM = "instagram/rankLikes/big"
        TWITTER = "twitter/rankLikes/big"
        TWITCH = "twitch/rankViewer/big"
        TIKTOK = "tiktok/rankLikes/big"

        DISPLAY = {
            YOUTUBE: "youtube",
            INSTAGRAM: "instagram",
            TWITTER: "twitter",
            TWITCH: "twitch",
            TIKTOK: "tiktok"
        }


class Milestone(object):

    def __init__(self, currentSubs, expected_time, surpassed, channel, artist):

        self.current_subs = currentSubs
        self.expected_time = expected_time
        self.surpassed = surpassed

        self.channel = channel
        self.artist = artist

    def __repr__(self):
        return f"<nindo.api.Milestone '{self.current_subs}'>"


class Brand(object):

    def __init__(self, id, name, url, branch, color):

        self.id = id
        self.name = name
        self.url = url
        self.branch = branch
        self.color = color

    def __repr__(self):

        return f"<nindo.api.Brand '{self.id}'>"